/* (C) 2016-2021 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Component, Input, OnDestroy} from "@angular/core";
import {PrinterAction, RepetierState, ServerAction, ServerState} from "binding";
import {RSModal} from "./modals/modal.service";
import {NgRedux, select} from "@angular-redux/store";
import {MessageState, PrintersState, PrinterState} from "binding";
import {Observable, SubscriptionLike} from "rxjs";
import {RSUIHelper} from "./services/ui-helper.service";
import {Router} from "@angular/router";
import {SubscriptionLike as ISubscription} from "rxjs/internal/types";

@Component({
  selector: "rs-headline",
  template: `
    <h1 style="font-weight: 400;display: flex;flex-wrap: nowrap;justify-content: space-around;">
    <span  *ngIf="wideScreen && printer && printer.online===1" style="font-size:90%;margin-left:10px">
        <span style="padding-right:10px;flex-grow: 0" *ngIf="printer.extruders.length>0 && printer.config.extruders[printer.activeExtruderId].supportTemperature">
              <rs-button class="h1button" (hit)="ui.setExtruderTemp(printer.activeExtruderId)">
                <i [class]="printerAction.extruderIcon(printer.slug,printer.activeExtruderId)" class="fw text-primary" style="display: inline-block;position: relative;top: -7px;"></i>
                  <span style="font-size:15px"><span style="display:inline-block;line-height:17px">{{printer.extruders[printer.activeExtruderId].currentTemp | fixed:1}}°C<br>{{printer.extruders[printer.activeExtruderId].tempSet | fixed:0}}°C</span></span>
              </rs-button>
        </span>
        <span class="" *ngIf="printer.heatedBeds.length>0" style="flex-grow: 0">
              <rs-button class="h1button" (hit)="ui.setBedTemp(0)">
                <i class="rs rs-heated-bed fw text-primary" style="display: inline-block;position: relative;top: -7px;"></i>
                  <span style="font-size:15px"><span style="display:inline-block;line-height:17px">{{printer.heatedBeds[0].currentTemp | fixed:1}}°C<br>{{printer.heatedBeds[0].tempSet | fixed:0}}°C</span></span>
              </rs-button>
        </span>
        <span class="" *ngIf="printer.heatedChambers.length>0" style="padding-left:8px;flex-grow: 0">
              <rs-button class="h1button" (hit)="ui.setChamberTemp(0)">
                <i class="rs rs-chamber fw text-primary" style="display: inline-block;position: relative;top: -7px;"></i>
                  <span style="font-size:15px"><span style="display:inline-block;line-height:17px">{{printer.heatedChambers[0].currentTemp | fixed:1}}°C<br>{{printer.heatedChambers[0].tempSet | fixed:0}}°C</span></span>
              </rs-button>
        </span>
     </span>
     <span *ngIf="(messages$|async).length" style="flex-grow: 0;margin-left: 10px">
      <rs-button class="h1button" (hit)="showMessages()" style="font-size:90%"><i class="fa fa-envelope text-primary"><span class="counter">{{(messages$|async).length}}</span></i></rs-button>
    </span>
    <span (rsClick)="goPrinterList()" style="flex-grow: 1"><ng-content *ngIf="showFunction"></ng-content>
    <span *ngIf="!showFunction" style="flex-grow: 1"><i class="rs rs-printer"></i> {{printers.printers[printers.active].name}}</span>
    </span></h1>`
})
export class RSHeadlineComponent implements OnDestroy {
  @select("messages") messages$: Observable<MessageState[]>;
  @select("server") server$: Observable<ServerState>;
  printer: PrinterState;
  printers: PrintersState;
  @select("printers") printers$: Observable<PrintersState>;
  wideScreen: Boolean = false;
  showFunction: Boolean = true;
  private printersSubscription: SubscriptionLike;
  private serverSubscription: ISubscription;
  serverState: ServerState = undefined;
  intervalStop: any;
  @Input() nameSwitchable = true;

  constructor(public ui: RSUIHelper,
              public printerAction: PrinterAction,
              public modal: RSModal,
              public router: Router,
              private serverAction: ServerAction,
              private redux: NgRedux<RepetierState>) {
    this.serverAction.initSetting("gui", {"autopopupMessages": 1, "emergencyIcon": 0, "noAnimation": 1, "printerNamePos": 1});
    this.wideScreen = window.innerWidth >= 800;
    this.printersSubscription = this.printers$.subscribe((p: PrintersState) => {
      this.printers = p;
      if (p.active) {
        this.printer = p.printers[p.active];
      } else {
        this.printer = undefined;
      }
    });
    this.serverSubscription = this.server$.subscribe((s: ServerState) => {
      this.serverState = s;
      if (this.serverState.settings["gui"] && this.serverState.settings["gui"].printerNamePos !== 1) {
        this.showFunction = true;
      }
    });
    this.intervalStop = setInterval(() => {
      if (this.serverState.settings["gui"] && this.serverState.settings["gui"].printerNamePos === 1 && this.printers.sortedPrinters.length > 1 && this.nameSwitchable && this.printer) {
        this.showFunction = !this.showFunction;
      }
    }, 2000);
  }

  goPrinterList(): void {
    if (this.printers.sortedPrinters.length > 1 && this.nameSwitchable) {
      this.ui.selectPrinter();
      // this.router.navigateByUrl("/printer-list");
    }
  }

  showMessages(): void {
    this.router.navigateByUrl("/messages");
  }

  ngOnDestroy(): void {
    clearInterval(this.intervalStop);
    this.printersSubscription.unsubscribe();
    this.serverSubscription.unsubscribe();
  }
}
